@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2008 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2008, 2010, 2011, 2014, 2016 Graeme Gott <graeme@gottcode.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,32 +20,44 @@
 #ifndef BOARD_H
 #define BOARD_H
 
-#include <QGLWidget>
-#include <QHash>
+#include "graphics_layer.h"
+class AppearanceDialog;
+class Message;
 class Overview;
 class Piece;
 class Tile;
 
-class Board : public QGLWidget
+#include <QHash>
+#include <QOpenGLWidget>
+class QOpenGLTexture;
+
+#include <random>
+
+class Board : public QOpenGLWidget
 {
 	Q_OBJECT
+
+	struct Region
+	{
+		VertexArray fill;
+		VertexArray border;
+	};
+
 public:
 	Board(QWidget* parent = 0);
 	~Board();
 
+	Piece* findCollidingPiece(Piece* piece) const;
 	void removePiece(Piece* piece);
 
-	QList<Piece*> findCollidingPieces(Piece* piece) const;
-	Piece* findCollidingPiece(Piece* piece) const;
-
-	int id() const
-		{ return m_id; }
-	int tileSize() const
-		{ return m_tile_size; }
-	int margin() const
-		{ return 10.0f / m_scale; }
-	bool bordersVisible() const
-		{ return m_borders_visible; }
+	int id() const;
+	int margin() const;
+	QRect marginRect(const QRect& rect) const;
+	int randomInt(int max);
+	float tileTextureSize() const;
+	const QPointF* corners(int rotation) const;
+	void setAppearance(const AppearanceDialog& dialog);
+	void updateSceneRectangle(Piece* piece);
 
 public slots:
 	void newGame(const QString& image, int difficulty);
@@ -55,19 +67,18 @@ public slots:
 	void zoomIn();
 	void zoomOut();
 	void zoomFit();
-	void zoom(int value);
+	void zoom(int level);
 	void toggleOverview();
-	void toggleBorders();
 
 signals:
+	void completionChanged(int value);
 	void overviewToggled(bool visible);
-	void bordersToggled(bool visible);
-	void statusMessage(const QString& message);
 	void retrievePiecesAvailable(bool available);
+	void showMessage(const QString& message, int timeout = 0);
+	void clearMessage();
 	void zoomInAvailable(bool available);
 	void zoomOutAvailable(bool available);
-	void zoomChanged(int zoom);
-	void zoomRangeChanged(int min, int max);
+	void zoomChanged(int level, float factor);
 	void finished();
 
 protected:
@@ -92,39 +103,43 @@ private:
 	void rotatePiece();
 	void selectPieces();
 
+	void drawArray(const Region& region, const QColor& fill, const QColor& border);
 	void loadImage();
-	void generateSuccessImage();
 	void updateCursor();
 	QPoint mapCursorPosition() const;
 	QPoint mapPosition(const QPoint& position) const;
-	void draw(Tile* tile, const QPoint& pos, float depth) const;
 	void updateCompleted();
-	Tile* tileAt(const QPoint& pos, bool include_active = true) const;
-	Tile* tileUnderCursor(bool include_active = true);
+	void updateArray(Region& region, const QRect& rect, int z);
+	void updateSceneRectangle();
+	void updateStatusMessage(const QString& message);
+	Piece* pieceUnderCursor();
+	int pieceCount();
 	void finishGame();
 	void cleanup();
 
+private:
 	int m_id;
-	int m_difficulty;
-	bool m_letterbox;
-	bool m_borders_visible;
+	bool m_load_bevels;
 	QString m_image_path;
-	int m_image_width;
-	int m_image_height;
-	int m_tile_size;
-	int m_bumpmap_size;
 	Overview* m_overview;
+	Message* m_message;
+	bool m_has_bevels;
+	bool m_has_shadows;
 
-	GLuint m_success;
-	GLuint m_image;
-	GLuint m_bumpmap;
+	QOpenGLTexture* m_image;
+	QOpenGLTexture* m_bumpmap_image;
+	QOpenGLTexture* m_shadow_image;
 	float m_image_ts;
-	float m_bumpmap_ts;
 	QPointF m_corners[4][4];
-	QSize m_success_size;
+	Region m_scene_array;
+	Region m_selection_array;
 
+	int m_columns;
+	int m_rows;
 	QList<Piece*> m_pieces;
-	QHash<Piece*, Tile*> m_active_tiles;
+	QList<Piece*> m_active_pieces;
+	QList<Piece*> m_selected_pieces;
+	QRect m_scene;
 	int m_total_pieces;
 	int m_completed;
 
@@ -132,8 +147,6 @@ private:
 	QPoint m_cursor_pos;
 	QPoint m_select_pos;
 	int m_scale_level;
-	int m_scale_level_min;
-	int m_scale_level_max;
 	float m_scale;
 	bool m_scrolling;
 	bool m_selecting;
@@ -141,6 +154,40 @@ private:
 
 	int m_action_key;
 	Qt::MouseButton m_action_button;
+
+	std::mt19937 m_random;
 };
 
-#endif // BOARD_H
+
+inline int Board::id() const
+{
+	return m_id;
+}
+
+inline int Board::margin() const
+{
+	return 16;
+}
+
+inline QRect Board::marginRect(const QRect& rect) const
+{
+	return rect.adjusted(-margin(), -margin(), margin(), margin());
+}
+
+inline int Board::randomInt(int max)
+{
+	std::uniform_int_distribution<int> dis(0, max - 1);
+	return dis(m_random);
+}
+
+inline float Board::tileTextureSize() const
+{
+	return m_image_ts;
+}
+
+inline const QPointF* Board::corners(int rotation) const
+{
+	return m_corners[rotation];
+}
+
+#endif
