@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2008 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2008, 2011, 2012, 2016 Graeme Gott <graeme@gottcode.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,38 +20,37 @@
 #ifndef THUMBNAIL_LOADER_H
 #define THUMBNAIL_LOADER_H
 
+struct Thumbnail;
+
+#include <QAtomicInt>
 #include <QMutex>
 #include <QThread>
-class QPixmap;
+class QListWidget;
+class QListWidgetItem;
 
-class ThumbnailLoader : public QThread {
+class ThumbnailLoader : public QThread
+{
 	Q_OBJECT
+
+	ThumbnailLoader(QObject* parent = 0);
 public:
-	ThumbnailLoader();
+	~ThumbnailLoader();
 
-	static QPixmap loadingIcon();
-
-	void add(const QString& file, const QString& preview);
-	void stop();
-
-public slots:
-	void clear();
+	static QListWidgetItem* createItem(const QString& image, const QString& text, QListWidget* list, qreal pixelratio);
 
 signals:
-	void generated(const QString& file);
+	void loaded(const Thumbnail& details);
 
 protected:
 	virtual void run();
 
+private slots:
+	void imageLoaded(const Thumbnail& details);
+
 private:
 	bool m_done;
-	struct Thumbnail {
-		QString file;
-		QString preview;
-	};
-	QList<Thumbnail> m_thumbnails;
-	QMutex m_done_mutex;
-	QMutex m_thumbnails_mutex;
+	QList<Thumbnail> m_details;
+	QMutex m_mutex;
 };
 
 #endif
